@@ -37,7 +37,6 @@ except ImportError:
     pass
 
 from .asteval import Interpreter
-from .astutils import NameFinder
 from .parameter import Parameter, Parameters
 
 # use locally modified version of uncertainties package
@@ -54,32 +53,28 @@ def asteval_with_uncertainties(*vals, **kwargs):
     _obj = kwargs.get('_obj', None)
     _pars = kwargs.get('_pars', None)
     _names = kwargs.get('_names', None)
-    _asteval = kwargs.get('_asteval', None)
-    if (_obj is None or
-        _pars is None or
-        _names is None or
-        _asteval is None or
-        _obj.ast is None):
+    _asteval = _pars._asteval
+    if (_obj is None or  _pars is None or _names is None or
+        _asteval is None or _obj._expr_ast is None):
         return 0
     for val, name in zip(vals, _names):
         _asteval.symtable[name] = val
-    return _asteval.eval(_obj.ast)
+    return _asteval.eval(_obj._expr_ast)
 
 wrap_ueval = uncertainties.wrap(asteval_with_uncertainties)
 
-def eval_stderr(obj, uvars, _names, _pars, _asteval):
+def eval_stderr(obj, uvars, _names, _pars):
     """evaluate uncertainty and set .stderr for a parameter `obj`
     given the uncertain values `uvars` (a list of uncertainties.ufloats),
     a list of parameter names that matches uvars, and a dict of param
     objects, keyed by name.
 
-    This uses the uncertainties package wrapped function to evaluate
-    the uncertainty for an arbitrary expression (in obj.ast) of parameters.
+    This uses the uncertainties package wrapped function to evaluate the
+    uncertainty for an arbitrary expression (in obj._expr_ast) of parameters.
     """
-    if not isinstance(obj, Parameter) or not hasattr(obj, 'ast'):
+    if not isinstance(obj, Parameter) or getattr(obj, '_expr_ast', None) is None:
         return
-    uval = wrap_ueval(*uvars, _obj=obj, _names=_names,
-                      _pars=_pars, _asteval=_asteval)
+    uval = wrap_ueval(*uvars, _obj=obj, _names=_names, _pars=_pars)
     try:
         obj.stderr = uval.std_dev()
     except:
@@ -96,14 +91,6 @@ class MinimizerException(Exception):
         return "\n%s" % (self.msg)
 
 
-def check_ast_errors(error):
-    """check for errors derived from asteval, raise MinimizerException"""
-    if len(error) > 0:
-        for err in error:
-            msg = '%s: %s' % (err.get_error())
-            return msg
-    return None
-
 def _differential_evolution(func, x0, **kwds):
     """
     A wrapper for differential_evolution that can be used with scipy.minimize
@@ -124,8 +111,8 @@ SCALAR_METHODS = {'nelder': 'Nelder-Mead',
                   'cg': 'CG',
                   'bfgs': 'BFGS',
                   'newton': 'Newton-CG',
-                  'lbfgs': 'L-BFGS-B',
-                  'l-bfgs':'L-BFGS-B',
+                  'lbfgsb': 'L-BFGS-B',
+                  'l-bfgsb':'L-BFGS-B',
                   'tnc': 'TNC',
                   'cobyla': 'COBYLA',
                   'slsqp': 'SLSQP',
@@ -134,6 +121,30 @@ SCALAR_METHODS = {'nelder': 'Nelder-Mead',
                   'differential_evolution': 'differential_evolution'}
 
 
+class MinimizerResult(object):
+    """ The result of a minimization.
+
+    Attributes
+    ----------
+    params : Parameters
+        The best-fit parameters
+    success : bool
+        Whether the minimization was successful
+    status : int
+        Termination status of the optimizer. Its value depends on the
+        underlying solver. Refer to `message` for details.
+
+    Notes
+    -----
+    additional attributes not listed above depending of the
+    specific solver. Since this class is essentially a subclass of dict
+    with attribute accessors, one can see which attributes are available
+    using the `keys()` method.
+    """
+    def __init__(self, **kws):
+        for key, val in kws.items():
+            setattr(self, key, val)
+
 class Minimizer(object):
     """A general minimizer for curve fitting"""
     err_nonparam = ("params must be a minimizer.Parameters() instance or list "
@@ -202,8 +213,8 @@ class Minimizer(object):
         self.nfev = 0
         self.nfree = 0
         self.ndata = 0
-        self.nvarys = 0
         self.ier = 0
+        self._abort = False
         self.success = True
         self.errorbars = False
         self.message = None
@@ -212,16 +223,9 @@ class Minimizer(object):
         self.redchi = None
         self.covar = None
         self.residual = None
-        self.var_map = []
-        self.vars = []
-        self.params = []
-        self.updated = []
+
+        self.params = params
         self.jacfcn = None
-        self.asteval = Interpreter()
-        self.namefinder = NameFinder()
-        self.__prepared = False
-        self.__set_params(params)
-        # self.prepare_fit()
 
     @property
     def values(self):
@@ -232,42 +236,7 @@ class Minimizer(object):
             Parameter values in a simple dictionary.
         """
 
-        return dict([(name, p.value) for name, p in self.params.items()])
-
-    def __update_paramval(self, name):
-        """
-        update parameter value, including setting bounds.
-        For a constrained parameter (one with an expr defined),
-        this first updates (recursively) all parameters on which
-        the parameter depends (using the 'deps' field).
-       """
-        # Has this param already been updated?
-        # if this is called as an expression dependency,
-        # it may have been!
-        if self.updated[name]:
-            return
-        par = self.params[name]
-        if getattr(par, 'expr', None) is not None:
-            if getattr(par, 'ast', None) is None:
-                par.ast = self.asteval.parse(par.expr)
-            if par.deps is not None:
-                for dep in par.deps:
-                    self.__update_paramval(dep)
-            par.value = self.asteval.run(par.ast)
-            out = check_ast_errors(self.asteval.error)
-            if out is not None:
-                self.asteval.raise_exception(None)
-        self.asteval.symtable[name] = par.value
-        self.updated[name] = True
-
-    def update_constraints(self):
-        """
-        Update all constrained parameters, checking that dependencies are
-        evaluated as needed.
-        """
-        self.updated = dict([(name, False) for name in self.params])
-        for name in self.params:
-            self.__update_paramval(name)
+        return dict([(name, p.value) for name, p in self.result.params.items()])
 
     def __residual(self, fvars):
         """
@@ -278,58 +247,55 @@ class Minimizer(object):
         calculate the residual.
         """
         # set parameter values
-        for varname, val in zip(self.var_map, fvars):
-            # self.params[varname].value = val
-            par = self.params[varname]
-            par.value = par.from_internal(val)
-        self.nfev = self.nfev + 1
+        if self._abort:
+            return None
+        params = self.result.params
+        for name, val in zip(self.result.var_names, fvars):
+            params[name].value = params[name].from_internal(val)
+        self.result.nfev = self.result.nfev + 1
 
-        self.update_constraints()
-        out = self.userfcn(self.params, *self.userargs, **self.userkws)
+        params.update_constraints()
+        out = self.userfcn(params, *self.userargs, **self.userkws)
         if callable(self.iter_cb):
-            self.iter_cb(self.params, self.nfev, out,
-                         *self.userargs, **self.userkws)
-        return out
+            abort = self.iter_cb(params, self.result.nfev, out,
+                                 *self.userargs, **self.userkws)
+            self._abort = self._abort or abort
+        if not self._abort:
+            return np.asarray(out).ravel()
 
     def __jacobian(self, fvars):
         """
         analytical jacobian to be used with the Levenberg-Marquardt
 
         modified 02-01-2012 by Glenn Jones, Aberystwyth University
+        modified 06-29-2015 M Newville to apply gradient scaling
+               for bounded variables (thanks to JJ Helmus, N Mayorov)
         """
-        for varname, val in zip(self.var_map, fvars):
-            # self.params[varname].value = val
-            self.params[varname].value = self.params[varname].from_internal(val)
+        pars  = self.result.params
+        grad_scale = ones_like(fvars)
+        for ivar, name in enumerate(self.result.var_names):
+            val = fvars[ivar]
+            pars[name].value = pars[name].from_internal(val)
+            grad_scale[ivar] = pars[name].scale_gradient(val)
 
-        self.nfev = self.nfev + 1
-        self.update_constraints()
-        # computing the jacobian
-        return self.jacfcn(self.params, *self.userargs, **self.userkws)
-
-    def __set_params(self, params):
-        """ set internal self.params from a Parameters object or
-        a list/tuple of Parameters"""
-        if params is None or isinstance(params, Parameters):
-            self.params = params
-        elif isinstance(params, (list, tuple)):
-            _params = Parameters()
-            for _par in params:
-                if not isinstance(_par, Parameter):
-                    raise MinimizerException(self.err_nonparam)
-                else:
-                    _params[_par.name] = _par
-            self.params = _params
+        self.result.nfev = self.result.nfev + 1
+        pars.update_constraints()
+        # compute the jacobian for "internal" unbounded variables,
+        # the rescale for bounded "external" variables.
+        jac = self.jacfcn(pars, *self.userargs, **self.userkws)
+        if self.col_deriv:
+            jac = (jac.transpose()*grad_scale).transpose()
         else:
-            raise MinimizerException(self.err_nonparam)
+            jac = jac*grad_scale
+        return jac
 
-    def penalty(self, params):
+    def penalty(self, fvars):
         """
         Penalty function for scalar minimizers:
 
         Parameters
         ----------
-        params : lmfit.parameter.Parameters object
-            The model parameters
+        fvars : array of values for the variable parameters
 
         Returns
         -------
@@ -337,52 +303,56 @@ class Minimizer(object):
             The user evaluated user-supplied objective function. If the
             objective function is an array, return the array sum-of-squares
         """
-        r = self.__residual(params)
+        r = self.__residual(fvars)
         if isinstance(r, ndarray):
             r = (r*r).sum()
         return r
 
     def prepare_fit(self, params=None):
         """
-        Prepares parameters for fitting
+        Prepares parameters for fitting,
+        return array of initial values
         """
         # determine which parameters are actually variables
         # and which are defined expressions.
-        if params is None and self.params is not None and self.__prepared:
-            return
-        if params is not None and self.params is None:
-            self.__set_params(params)
-        self.nfev = 0
-        self.var_map = []
-        self.vars = []
-        for name, par in self.params.items():
+        result = self.result = MinimizerResult()
+        if params is not None:
+            self.params = params
+        if isinstance(self.params, Parameters):
+            result.params = deepcopy(self.params)
+        elif isinstance(self.params, (list, tuple)):
+            result.params = Parameters()
+            for par in self.params:
+                if not isinstance(par, Parameter):
+                    raise MinimizerException(self.err_nonparam)
+                else:
+                    result.params[par.name] = par
+        elif self.params is None:
+            raise MinimizerException(self.err_nonparam)
+
+        # determine which parameters are actually variables
+        # and which are defined expressions.
+
+        result.var_names = [] # note that this *does* belong to self...
+        result.init_vals = []
+        result.params.update_constraints()
+        result.nfev = 0
+        result.errorbars = False
+        result.aborted = False
+        for name, par in self.result.params.items():
+            par.stderr = None
+            par.correl = None
             if par.expr is not None:
-                par.ast = self.asteval.parse(par.expr)
-                check_ast_errors(self.asteval.error)
                 par.vary = False
-                par.deps = []
-                self.namefinder.names = []
-                self.namefinder.generic_visit(par.ast)
-                for symname in self.namefinder.names:
-                    if (symname in self.params and
-                        symname not in par.deps):
-                        par.deps.append(symname)
-            elif par.vary:
-                self.var_map.append(name)
-                self.vars.append(par.setup_bounds())
+            if par.vary:
+                result.var_names.append(name)
+                result.init_vals.append(par.setup_bounds())
 
-            self.asteval.symtable[name] = par.value
             par.init_value = par.value
             if par.name is None:
                 par.name = name
-
-        self.nvarys = len(self.vars)
-
-        # now evaluate make sure initial values
-        # are used to set values of the defined expressions.
-        # this also acts as a check of expression syntax.
-        self.update_constraints()
-        self.__prepared = True
+        result.nvarys = len(result.var_names)
+        return result
 
     def unprepare_fit(self):
         """
@@ -391,11 +361,7 @@ class Minimizer(object):
 
         removes ast compilations of constraint expressions
         """
-        self.__prepared = False
-        self.params = deepcopy(self.params)
-        for par in self.params.values():
-            if hasattr(par, 'ast'):
-                delattr(par, 'ast')
+        pass
 
     @deprecate(message='    Deprecated in lmfit 0.8.2, use scalar_minimize '
                        'and method=\'L-BFGS-B\' instead')
@@ -410,27 +376,8 @@ class Minimizer(object):
             scipy.optimize.lbfgsb.fmin_l_bfgs_b function.
 
         """
+        raise NotImplementedError("use scalar_minimize(method='L-BFGS-B')")
 
-        self.prepare_fit()
-        lb_kws = dict(factr=1000.0, approx_grad=True, m=20,
-                      maxfun=2000 * (self.nvarys + 1),
-                      )
-        lb_kws.update(self.kws)
-        lb_kws.update(kws)
-
-        xout, fout, info = scipy_lbfgsb(self.penalty, self.vars, **lb_kws)
-        self.nfev = info['funcalls']
-        self.message = info['task']
-        self.chisqr = self.residual = self.__residual(xout)
-        self.ndata = 1
-        self.nfree = 1
-        if isinstance(self.residual, ndarray):
-            self.chisqr = (self.chisqr**2).sum()
-            self.ndata = len(self.residual)
-            self.nfree = self.ndata - self.nvarys
-        self.redchi = self.chisqr/self.nfree
-        self.unprepare_fit()
-        return
 
     @deprecate(message='    Deprecated in lmfit 0.8.2, use scalar_minimize '
                        'and method=\'Nelder-Mead\' instead')
@@ -443,29 +390,9 @@ class Minimizer(object):
         kws : dict
             Minimizer options to pass to the scipy.optimize.fmin minimizer.
         """
+        raise NotImplementedError("use scalar_minimize(method='Nelder-Mead')")
 
-        self.prepare_fit()
-        fmin_kws = dict(full_output=True, disp=False, retall=True,
-                        ftol=1.e-4, xtol=1.e-4,
-                        maxfun=5000 * (self.nvarys + 1))
-
-        fmin_kws.update(kws)
-
-        ret = scipy_fmin(self.penalty, self.vars, **fmin_kws)
-        xout, fout, niter, funccalls, warnflag, allvecs = ret
-        self.nfev = funccalls
-        self.chisqr = self.residual = self.__residual(xout)
-        self.ndata = 1
-        self.nfree = 1
-        if isinstance(self.residual, ndarray):
-            self.chisqr = (self.chisqr**2).sum()
-            self.ndata = len(self.residual)
-            self.nfree = self.ndata - self.nvarys
-        self.redchi = self.chisqr/self.nfree
-        self.unprepare_fit()
-        return
-
-    def scalar_minimize(self, method='Nelder-Mead', **kws):
+    def scalar_minimize(self, method='Nelder-Mead', params=None, **kws):
         """
         Use one of the scalar minimization methods from
         scipy.optimize.minimize.
@@ -487,6 +414,8 @@ class Minimizer(object):
                 'SLSQP'
                 'differential_evolution'
 
+        params : Parameters, optional
+           Parameters to use as starting points.
         kws : dict, optional
             Minimizer options pass to scipy.optimize.minimize.
 
@@ -508,10 +437,13 @@ class Minimizer(object):
         """
         if not HAS_SCALAR_MIN:
             raise NotImplementedError
-        self.prepare_fit()
+
+        result = self.prepare_fit(params=params)
+        vars   = result.init_vals
+        params = result.params
 
         fmin_kws = dict(method=method,
-                        options={'maxiter': 1000 * (self.nvarys + 1)})
+                        options={'maxiter': 1000 * (len(vars) + 1)})
         fmin_kws.update(self.kws)
         fmin_kws.update(kws)
 
@@ -532,31 +464,43 @@ class Minimizer(object):
 
         if method == 'differential_evolution':
             fmin_kws['method'] = _differential_evolution
-            pars = self.params
-            bounds = [(pars[par].min, pars[par].max) for par in pars]
+            bounds = [(par.min, par.max) for par in params.values()]
             if not np.all(np.isfinite(bounds)):
                 raise ValueError('With differential evolution finite bounds '
                                  'are required for each parameter')
-            bounds = [(-np.pi / 2., np.pi / 2.)] * len(self.vars)
+            bounds = [(-np.pi / 2., np.pi / 2.)] * len(vars)
             fmin_kws['bounds'] = bounds
 
-        ret = scipy_minimize(self.penalty, self.vars, **fmin_kws)
-        xout = ret.x
-        self.message = ret.message
+            # in scipy 0.14 this can be called directly from scipy_minimize
+            # When minimum scipy is 0.14 the following line and the else
+            # can be removed.
+            ret = _differential_evolution(self.penalty, vars, **fmin_kws)
+        else:
+            ret = scipy_minimize(self.penalty, vars, **fmin_kws)
 
-        self.nfev = ret.nfev
-        self.chisqr = self.residual = self.__residual(xout)
-        self.ndata = 1
-        self.nfree = 1
-        if isinstance(self.residual, ndarray):
-            self.chisqr = (self.chisqr**2).sum()
-            self.ndata = len(self.residual)
-            self.nfree = self.ndata - self.nvarys
-        self.redchi = self.chisqr / self.nfree
-        self.unprepare_fit()
-        return ret.success
+        result.aborted = self._abort
+        self._abort = False
 
-    def leastsq(self, **kws):
+        for attr in dir(ret):
+            if not attr.startswith('_'):
+                setattr(result, attr, getattr(ret, attr))
+
+        result.chisqr = result.residual = self.__residual(ret.x)
+        result.nvarys = len(vars)
+        result.ndata = 1
+        result.nfree = 1
+        if isinstance(result.residual, ndarray):
+            result.chisqr = (result.chisqr**2).sum()
+            result.ndata = len(result.residual)
+            result.nfree = result.ndata - result.nvarys
+        result.redchi = result.chisqr / result.nfree
+        _log_likelihood = result.ndata * np.log(result.redchi)
+        result.aic = _log_likelihood + 2 * result.nvarys
+        result.bic = _log_likelihood + np.log(result.ndata) * result.nvarys
+
+        return result
+
+    def leastsq(self, params=None, **kws):
         """
         Use Levenberg-Marquardt minimization to perform a fit.
         This assumes that ModelParameters have been stored, and a function to
@@ -571,6 +515,8 @@ class Minimizer(object):
 
         Parameters
         ----------
+        params : Parameters, optional
+           Parameters to use as starting points.
         kws : dict, optional
             Minimizer options to pass to scipy.optimize.leastsq.
 
@@ -579,43 +525,55 @@ class Minimizer(object):
         success : bool
             True if fit was successful, False if not.
         """
-        self.prepare_fit()
-        lskws = dict(full_output=1, xtol=1.e-7, ftol=1.e-7,
-                     gtol=1.e-7, maxfev=2000*(self.nvarys+1), Dfun=None)
+        result = self.prepare_fit(params=params)
+        vars   = result.init_vals
+        nvars = len(vars)
+        lskws = dict(full_output=1, xtol=1.e-7, ftol=1.e-7, col_deriv=False,
+                     gtol=1.e-7, maxfev=2000*(nvars+1), Dfun=None)
 
         lskws.update(self.kws)
         lskws.update(kws)
 
+        self.col_deriv = False
         if lskws['Dfun'] is not None:
             self.jacfcn = lskws['Dfun']
+            self.col_deriv = lskws['col_deriv']
             lskws['Dfun'] = self.__jacobian
 
         # suppress runtime warnings during fit and error analysis
         orig_warn_settings = np.geterr()
         np.seterr(all='ignore')
-        lsout = scipy_leastsq(self.__residual, self.vars, **lskws)
+
+        lsout = scipy_leastsq(self.__residual, vars, **lskws)
         _best, _cov, infodict, errmsg, ier = lsout
+        result.aborted = self._abort
+        self._abort = False
 
-        self.residual = resid = infodict['fvec']
-        self.ier = ier
-        self.lmdif_message = errmsg
-        self.message = 'Fit succeeded.'
-        self.success = ier in [1, 2, 3, 4]
-
-        if ier == 0:
-            self.message = 'Invalid Input Parameters.'
+        result.residual = resid = infodict['fvec']
+        result.ier = ier
+        result.lmdif_message = errmsg
+        result.message = 'Fit succeeded.'
+        result.success = ier in [1, 2, 3, 4]
+        if result.aborted:
+            result.message = 'Fit aborted by user callback.'
+            result.success = False
+        elif ier == 0:
+            result.message = 'Invalid Input Parameters.'
         elif ier == 5:
-            self.message = self.err_maxfev % lskws['maxfev']
+            result.message = self.err_maxfev % lskws['maxfev']
         else:
-            self.message = 'Tolerance seems to be too small.'
+            result.message = 'Tolerance seems to be too small.'
 
-        self.nfev = infodict['nfev']
-        self.ndata = len(resid)
+        result.ndata = len(resid)
 
-        sum_sqr = (resid**2).sum()
-        self.chisqr = sum_sqr
-        self.nfree = (self.ndata - self.nvarys)
-        self.redchi = sum_sqr / self.nfree
+        result.chisqr = (resid**2).sum()
+        result.nfree = (result.ndata - nvars)
+        result.redchi = result.chisqr / result.nfree
+        _log_likelihood = result.ndata * np.log(result.redchi)
+        result.aic = _log_likelihood + 2 * nvars
+        result.bic = _log_likelihood + np.log(result.ndata) * nvars
+
+        params = result.params
 
         # need to map _best values to params, then calculate the
         # grad for the variable parameters
@@ -631,45 +589,44 @@ class Minimizer(object):
         if len(np.shape(grad)) == 0:
             grad = np.array([grad])
 
-        for ivar, varname in enumerate(self.var_map):
-            par = self.params[varname]
-            grad[ivar] = par.scale_gradient(_best[ivar])
-            vbest[ivar] = par.value
+        for ivar, name in enumerate(result.var_names):
+            grad[ivar] = params[name].scale_gradient(_best[ivar])
+            vbest[ivar] = params[name].value
 
         # modified from JJ Helmus' leastsqbound.py
         infodict['fjac'] = transpose(transpose(infodict['fjac']) /
                                      take(grad, infodict['ipvt'] - 1))
-        rvec = dot(triu(transpose(infodict['fjac'])[:self.nvarys, :]),
-                   take(eye(self.nvarys), infodict['ipvt'] - 1, 0))
+        rvec = dot(triu(transpose(infodict['fjac'])[:nvars, :]),
+                   take(eye(nvars), infodict['ipvt'] - 1, 0))
         try:
-            self.covar = inv(dot(transpose(rvec), rvec))
+            result.covar = inv(dot(transpose(rvec), rvec))
         except (LinAlgError, ValueError):
-            self.covar = None
+            result.covar = None
 
         has_expr = False
-        for par in self.params.values():
+        for par in params.values():
             par.stderr, par.correl = 0, None
             has_expr = has_expr or par.expr is not None
 
         # self.errorbars = error bars were successfully estimated
-        self.errorbars = self.covar is not None
-
-        if self.covar is not None:
+        result.errorbars = (result.covar is not None)
+        if result.aborted:
+            result.errorbars = False
+        if result.errorbars:
             if self.scale_covar:
-                self.covar = self.covar * sum_sqr / self.nfree
-
-            for ivar, varname in enumerate(self.var_map):
-                par = self.params[varname]
-                par.stderr = sqrt(self.covar[ivar, ivar])
+                result.covar *= result.redchi
+            for ivar, name in enumerate(result.var_names):
+                par = params[name]
+                par.stderr = sqrt(result.covar[ivar, ivar])
                 par.correl = {}
                 try:
-                    self.errorbars = self.errorbars and (par.stderr > 0.0)
-                    for jvar, varn2 in enumerate(self.var_map):
+                    result.errorbars = result.errorbars and (par.stderr > 0.0)
+                    for jvar, varn2 in enumerate(result.var_names):
                         if jvar != ivar:
-                            par.correl[varn2] = (self.covar[ivar, jvar] /
-                                 (par.stderr * sqrt(self.covar[jvar, jvar])))
+                            par.correl[varn2] = (result.covar[ivar, jvar] /
+                                 (par.stderr * sqrt(result.covar[jvar, jvar])))
                 except:
-                    self.errorbars = False
+                    result.errorbars = False
 
             uvars = None
             if has_expr:
@@ -679,26 +636,23 @@ class Minimizer(object):
                 #   re-evaluate contrained parameters to extract stderr
                 #   and then set Parameters back to best-fit value
                 try:
-                    uvars = uncertainties.correlated_values(vbest, self.covar)
+                    uvars = uncertainties.correlated_values(vbest, result.covar)
                 except (LinAlgError, ValueError):
                     uvars = None
-
                 if uvars is not None:
-                    for par in self.params.values():
-                        eval_stderr(par, uvars, self.var_map,
-                                    self.params, self.asteval)
+                    for par in params.values():
+                        eval_stderr(par, uvars, result.var_names, params)
                     # restore nominal values
-                    for v, nam in zip(uvars, self.var_map):
-                        self.asteval.symtable[nam] = v.nominal_value
+                    for v, nam in zip(uvars, result.var_names):
+                        params[nam].value = v.nominal_value
 
-        if not self.errorbars:
-            self.message = '%s. Could not estimate error-bars'  % self.message
+        if not result.errorbars:
+            result.message = '%s. Could not estimate error-bars'% result.message
 
         np.seterr(**orig_warn_settings)
-        self.unprepare_fit()
-        return self.success
+        return result
 
-    def minimize(self, method='leastsq'):
+    def minimize(self, method='leastsq', params=None, **kws):
         """
         Perform the minimization.
 
@@ -720,14 +674,20 @@ class Minimizer(object):
             'slsqp'                  -    Sequential Linear Squares Programming
             'differential_evolution' -    differential evolution
 
+        params : Parameters, optional
+            parameters to use as starting values
+
         Returns
         -------
-        success : bool
-            Whether the fit was successful.
+        result : MinimizerResult
+
+            MinimizerResult object contains updated params, fit statistics, etc.
+
         """
 
         function = self.leastsq
-        kwargs = {}
+        kwargs = {'params': params}
+        kwargs.update(kws)
 
         user_method = method.lower()
         if user_method.startswith('least'):
@@ -743,10 +703,8 @@ class Minimizer(object):
             function = self.fmin
         elif user_method.startswith('lbfgsb'):
             function = self.lbfgsb
-
         return function(**kwargs)
 
-
 def minimize(fcn, params, method='leastsq', args=None, kws=None,
              scale_covar=True, iter_cb=None, **fit_kws):
     """
@@ -813,5 +771,4 @@ def minimize(fcn, params, method='leastsq', args=None, kws=None,
     """
     fitter = Minimizer(fcn, params, fcn_args=args, fcn_kws=kws,
                        iter_cb=iter_cb, scale_covar=scale_covar, **fit_kws)
-    fitter.minimize(method=method)
-    return fitter
+    return fitter.minimize(method=method)
